using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Farbkreis
{
    public partial class Form1 : Form
    {
        private const int radius = 150;
        private Bitmap bitmap = new Bitmap(2 * radius + 1, 2 * radius + 1);

        public Form1()
        {
            InitializeComponent();
            buildBitmap();
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);
            e.Graphics.DrawImage(bitmap, 0, 0);
        }

        private void trackBar1_ValueChanged(object sender, EventArgs e)
        {
            buildBitmap();
            Invalidate();
        }

        private void buildBitmap()
        {
            for (int y = 0; y <= 2 * radius; y++)
            {
                for (int x = 0; x <= 2 * radius; x++)
                {
                    double dx = x - radius;
                    double dy = y - radius;
                    double distance = Math.Sqrt(dx * dx + dy * dy);
                    if (distance <= radius)
                    {
                        double phi = -Math.Atan2(dy, dx);
                        double phiDegrees = phi * 180.0 / Math.PI;
                        double r, g, b;
                        if (phiDegrees < -120.0)
                        {
                            r = 0.0;
                            g = -255.0 / 60.0 * (phiDegrees + 120.0);
                            b = 255.0;
                        }
                        else if (phiDegrees < -60.0)
                        {
                            r = 255.0 / 60.0 * (phiDegrees + 120.0);
                            g = 0.0;
                            b = 255.0;
                        }
                        else if (phiDegrees < 0.0)
                        {
                            r = 255.0;
                            g = 0.0;
                            b = -255.0 / 60.0 * phiDegrees;
                        }
                        else if (phiDegrees < 60.0)
                        {
                            r = 255.0;
                            g = 255.0 / 60.0 * phiDegrees;
                            b = 0.0;
                        }
                        else if (phiDegrees < 120.0)
                        {
                            r = -255.0 / 60.0 * (phiDegrees - 120.0);
                            g = 255.0;
                            b = 0.0;
                        }
                        else // phiDegrees >= 120.0
                        {
                            r = 0.0;
                            g = 255.0;
                            b = 255.0 / 60.0 * (phiDegrees - 120.0);
                        }
                        double ratio = distance / radius;
                        ratio = Math.Pow(ratio, 2.5 / 100.0 * trackBar1.Value + 0.5);
                        r = ratio * r + (1.0 - ratio) * 255.0;
                        g = ratio * g + (1.0 - ratio) * 255.0;
                        b = ratio * b + (1.0 - ratio) * 255.0;
                        Color c = Color.FromArgb((int)r, (int)g, (int)b);
                        bitmap.SetPixel(x, y, c);
                    }
                }
            }
        }
    }
}